'''
------------------------------------------------------------------------------
Name:         AttributeManager.py
Purpose:      Attributes are keyword type objects that may be associated
              with several types.
Author:       Jeremy Stark <jeremy@deximer.com>
Copyright:    (c) 2005 by Deximer
Licence:      GNU General Public Licence (GPL) Version 2 or later
------------------------------------------------------------------------------
'''

from sets import Set

from Products.CMFCore.utils import UniqueObject
from Products.CMFCore.utils import getToolByName
from Products.CMFCore import CMFCorePermissions
from AccessControl import ClassSecurityInfo

from Products.Archetypes.public import *
from Products.PortalTaxonomy.fields import AttributeField, CategoryField

class AttributeManager(BaseFolder, UniqueObject):
    ''' Control tool for portal_attributes
    '''
    security = ClassSecurityInfo()
    portal_type = meta_type = archetype_name = 'AttributeManager' 
    schema = BaseFolderSchema + Schema((),)

    def __init__(self):
        ''' This is a portal tool so just set the id and other object attributes
        '''
        BaseFolder.__init__(self, 'portal_attributes')
        self.setTitle('Attributes')

    def getAttributesFor(self, instance):
        ''' Get attributes for the instance type
            *** Need to write
        '''
        pass

    def getAvailableTypes(self):
        ''' Return types that are linked to Attributes
        '''
        result = []
        portalTypes = getToolByName(self, 'portal_types').listTypeInfo()
        pt = [t.Metatype() for t in portalTypes]
        types = [t for t in listTypes() if t['klass'].meta_type in pt]
        for type in types:
            schema = type['klass'].schema
            for field in schema.values():
                if field.getType()[-14:] == 'AttributeField' or \
                   field.getType()[-24:] == 'AttributeCollectionField':
                    result.append(type['klass'].meta_type)
        return DisplayList([(x, x) for x in result])

    def getVocab(self, instance):
        ''' return a vocab for a calling object
        '''
        attribs = []
        type = instance.getTypeInfo().Metatype()
        collections = self.listFolderContents()
        for c in collections:
            if type in c.type_restrictions:
                for a in c.listFolderContents():
                    attribs.append(a)
        return DisplayList([(a.UID(), a.title) for a in attribs])

    factory_type_information={
        'allowed_content_types':('AttributeCollection', ),
        'content_icon':'attributemanager.gif',
        'immediate_view':'folder_contents',
        'global_allow':0,
        'filter_content_types':1
        }

class AttributeCollection(BaseFolder):
    ''' Holds groups of attributes and assigns them to content types
    '''
    security = ClassSecurityInfo()
    portal_type = meta_type = archetype_name = 'AttributeCollection' 

    schema=BaseFolderSchema + Schema((
        LinesField( 'type_restrictions'
          , widget = MultiSelectionWidget(
                size = 10
              , label = "Type Restriction"
              , description = "Select the default type restrictions for "
                              "attributes in this collection"
            )
          , vocabulary = "getTypesVocab"
          , enforceVocabulary = True
        )
    ),)

    def getTypesVocab(self):
        ''' return types associated with AttributeManager
        '''
        return getToolByName(self, 'portal_attributes').getAvailableTypes()

    factory_type_information={
        'allowed_content_types':('Attribute',) ,
        'content_icon':'attributecollection.gif',
        'immediate_view':'base_view',
        'global_allow':0,
        'filter_content_types':1
        }

class Attribute(BaseContent):
    ''' Individualy selectable items contained in an attribute collection
    '''
    security = ClassSecurityInfo()
    portal_type = meta_type = archetype_name = 'Attribute' 

    schema=BaseContent.schema  + Schema((
        ImageField( 'image'
                  , original_size = (32, 32)
                  , sizes = {'small' : (16, 16)
                            ,'medium': (24, 24)
                            ,'large' : (32, 32)}
        ),
    ),)

    def getContent(self, review_state = 'published'):
        '''Returns published site content associated with this Attribute
        '''
        catalog = getToolByName(self, 'portal_catalog')
        return catalog.searchResults(
            getAttributes = self.UID()
          , review_state = review_state)

    factory_type_information={
        'allowed_content_types':() ,
        'content_icon':'attribute.gif',
        'immediate_view':'base_view',
        'global_allow':0,
        'filter_content_types':0
        }

registerType(AttributeManager)
registerType(AttributeCollection)
registerType(Attribute)

